@@ -48,7 +48,7 @@ final class UnsafeUtil {
       supportsUnsafeByteBufferOperations();
   private static final boolean HAS_UNSAFE_ARRAY_OPERATIONS = supportsUnsafeArrayOperations();
 
-  private static final long BYTE_ARRAY_BASE_OFFSET = arrayBaseOffset(byte[].class);
+  static final long BYTE_ARRAY_BASE_OFFSET = arrayBaseOffset(byte[].class);
   // Micro-optimization: we can assume a scale of 1 and skip the multiply
   // private static final long BYTE_ARRAY_INDEX_SCALE = 1;
 
@@ -72,7 +72,12 @@ final class UnsafeUtil {
 
   private static final long BUFFER_ADDRESS_OFFSET = fieldOffset(bufferAddressField());
 
-  private static final long STRING_VALUE_OFFSET = fieldOffset(stringValueField());
+  private static final int STRIDE = 8;
+  private static final int STRIDE_ALIGNMENT_MASK = STRIDE - 1;
+  private static final int BYTE_ARRAY_ALIGNMENT =
+      (int) (BYTE_ARRAY_BASE_OFFSET & STRIDE_ALIGNMENT_MASK);
+
+  static final boolean IS_BIG_ENDIAN = ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN;
 
   private UnsafeUtil() {}
 
@@ -85,6 +90,15 @@ final class UnsafeUtil {
   }
 
 
+  @SuppressWarnings("unchecked") // safe by method contract
+  static <T> T allocateInstance(Class<T> clazz) {
+    try {
+      return (T) UNSAFE.allocateInstance(clazz);
+    } catch (InstantiationException e) {
+      throw new IllegalStateException(e);
+    }
+  }
+
   static long objectFieldOffset(Field field) {
     return MEMORY_ACCESSOR.objectFieldOffset(field);
   }
@@ -228,7 +242,7 @@ final class UnsafeUtil {
   }
 
   static void copyMemory(byte[] src, long srcIndex, byte[] target, long targetIndex, long length) {
-    System.arraycopy(src, (int) srcIndex, target, (int) targetIndex, (int) length); 
+    System.arraycopy(src, (int) srcIndex, target, (int) targetIndex, (int) length);
   }
 
   static byte getByte(long address) {
@@ -255,33 +269,11 @@ final class UnsafeUtil {
     MEMORY_ACCESSOR.putLong(address, value);
   }
 
-  /**
-   * Gets the offset of the {@code address} field of the given direct {@link ByteBuffer}.
-   */
+  /** Gets the offset of the {@code address} field of the given direct {@link ByteBuffer}. */
   static long addressOffset(ByteBuffer buffer) {
     return MEMORY_ACCESSOR.getLong(buffer, BUFFER_ADDRESS_OFFSET);
   }
 
-  /**
-   * Returns a new {@link String} backed by the given {@code chars}. The char array should not
-   * be mutated any more after calling this function.
-   */
-  static String moveToString(char[] chars) {
-    if (STRING_VALUE_OFFSET == -1) {
-      // In the off-chance that this JDK does not implement String as we'd expect, just do a copy.
-      return new String(chars);
-    }
-    final String str;
-    try {
-      str = (String) UNSAFE.allocateInstance(String.class);
-    } catch (InstantiationException e) {
-      // This should never happen, but return a copy as a fallback just in case.
-      return new String(chars);
-    }
-    putObject(str, STRING_VALUE_OFFSET, chars);
-    return str;
-  }
-
   static Object getStaticObject(Field field) {
     return MEMORY_ACCESSOR.getStaticObject(field);
   }
@@ -397,10 +389,78 @@ final class UnsafeUtil {
     return field != null && field.getType() == long.class ? field : null;
   }
 
-  /** Finds the value field within a {@link String}. */
-  private static Field stringValueField() {
-    Field field = field(String.class, "value");
-    return field != null && field.getType() == char[].class ? field : null;
+  /**
+   * Returns the index of the first byte where left and right differ, in the range [0, 8]. If {@code
+   * left == right}, the result will be 8, otherwise less than 8.
+   *
+   * <p>This counts from the *first* byte, which may be the most or least significant byte depending
+   * on the system endianness.
+   */
+  private static int firstDifferingByteIndexNativeEndian(long left, long right) {
+    int n =
+        IS_BIG_ENDIAN
+            ? Long.numberOfLeadingZeros(left ^ right)
+            : Long.numberOfTrailingZeros(left ^ right);
+    return n >> 3;
+  }
+
+  /**
+   * Returns the lowest {@code index} such that {@code 0 <= index < length} and {@code left[leftOff
+   * + index] != right[rightOff + index]}. If no such value exists -- if {@code left} and {@code
+   * right} match up to {@code length} bytes from their respective offsets -- returns -1.
+   *
+   * <p>{@code leftOff + length} must be less than or equal to {@code left.length}, and the same for
+   * {@code right}.
+   */
+  static int mismatch(byte[] left, int leftOff, byte[] right, int rightOff, int length) {
+    if (leftOff < 0
+        || rightOff < 0
+        || length < 0
+        || leftOff + length > left.length
+        || rightOff + length > right.length) {
+      throw new IndexOutOfBoundsException();
+    }
+
+    int index = 0;
+    if (HAS_UNSAFE_ARRAY_OPERATIONS) {
+      int leftAlignment = (BYTE_ARRAY_ALIGNMENT + leftOff) & STRIDE_ALIGNMENT_MASK;
+
+      // Most CPUs handle getting chunks of bytes better on addresses that are a multiple of 4
+      // or 8.
+      // We walk one byte at a time until the left address, at least, is a multiple of 8.
+      // If the right address is, too, so much the better.
+      for (;
+          index < length && (leftAlignment & STRIDE_ALIGNMENT_MASK) != 0;
+          index++, leftAlignment++) {
+        if (left[leftOff + index] != right[rightOff + index]) {
+          return index;
+        }
+      }
+
+      // Stride!  Grab eight bytes at a time from left and right and check them for equality.
+
+      int strideLength = ((length - index) & ~STRIDE_ALIGNMENT_MASK) + index;
+      // strideLength is the point where we want to stop striding: it differs from index by
+      // a multiple of STRIDE, and it's the largest such number <= length.
+
+      for (; index < strideLength; index += STRIDE) {
+        long leftLongWord = getLong(left, BYTE_ARRAY_BASE_OFFSET + leftOff + index);
+        long rightLongWord = getLong(right, BYTE_ARRAY_BASE_OFFSET + rightOff + index);
+        if (leftLongWord != rightLongWord) {
+          // one of these eight bytes differ!  use a helper to find out which one
+          return index + firstDifferingByteIndexNativeEndian(leftLongWord, rightLongWord);
+        }
+      }
+    }
+
+    // If we were able to stride, there are at most STRIDE - 1 bytes left to compare.
+    // If we weren't, then this loop covers the whole thing.
+    for (; index < length; index++) {
+      if (left[leftOff + index] != right[rightOff + index]) {
+        return index;
+      }
+    }
+    return -1;
   }
 
   /**
@@ -412,14 +472,12 @@ final class UnsafeUtil {
   }
 
   /**
-   * Gets the field with the given name within the class, or {@code null} if not found. If found,
-   * the field is made accessible.
+   * Gets the field with the given name within the class, or {@code null} if not found.
    */
   private static Field field(Class<?> clazz, String fieldName) {
     Field field;
     try {
       field = clazz.getDeclaredField(fieldName);
-      field.setAccessible(true);
     } catch (Throwable t) {
       // Failed to access the fields.
       field = null;
@@ -500,9 +558,9 @@ final class UnsafeUtil {
     public abstract void putLong(long address, long value);
 
     public abstract Object getStaticObject(Field field);
-    
+
     public abstract void copyMemory(long srcOffset, byte[] target, long targetIndex, long length);
-    
+
     public abstract void copyMemory(byte[] src, long srcIndex, long targetOffset, long length);
   }
 
@@ -581,12 +639,12 @@ final class UnsafeUtil {
     public void putDouble(Object target, long offset, double value) {
       unsafe.putDouble(target, offset, value);
     }
-    
-    @Override 
+
+    @Override
     public void copyMemory(long srcOffset, byte[] target, long targetIndex, long length) {
       unsafe.copyMemory(null, srcOffset, target, BYTE_ARRAY_BASE_OFFSET + targetIndex, length);
     }
-    
+
     @Override
     public void copyMemory(byte[] src, long srcIndex, long targetOffset, long length) {
       unsafe.copyMemory(src, BYTE_ARRAY_BASE_OFFSET + srcIndex, null, targetOffset, length);
